import re
import pandas as pd

from collections import deque

from . import api
from .constants import Category


class BrowserError(Exception):
    """Generic Browser error."""


class Browser:

    root_category = Category.root

    def __init__(self, api_key: str = None):
        self.api_key = api_key
        self.browse()
        self.flags = {}

    def __repr__(self):
        subcats = len(self.childcategories)
        series = len(self.childseries)
        return f"<EIA Browser : {self.name} : Children {subcats} : Series {series}>"

    def browse(self, category_id: int = None):
        category_ids = self.parse_category_id(category_id)
        if len(category_ids) == 1:
            self.goto(category_ids[0])
        else:
            raise ValueError("Category id too vague, please be more specific.")

    def goto(self, category_id):
        category_id = category_id or self.root_category.value
        self._set_state(api.Category(category_id, self.api_key).to_dict())

    def _set_state(self, category_dict):
        self.name = category_dict["name"]
        self.childcategories = {
            d["name"]: d["category_id"] for d in category_dict["childcategories"]
        }
        self.childseries = category_dict["childseries"]
        self.category_id = category_dict["category_id"]
        self.parent_category_id = category_dict["parent_category_id"]

    def parse_category_id(self, category_id):
        cat = category_id  # >'.'<
        categories = []
        if not cat:
            categories = [self.root_category.value]
        elif category_id in self.childcategories.values():
            categories = [cat]
        elif category_id in self.childcategories:
            categories = [self.childcategories[cat]]
        if categories:
            return categories
        lookups = [
            # lambda x : self.childcategories[x] == cat,
            # lambda x : x == cat,
            lambda x: x.lower().startswith(cat.lower()),
            lambda x: re.search(cat, x, flags=re.I),
        ]
        for l in lookups:
            try:
                matches = list(filter(l, self.childcategories))
            except re.error:
                continue
            if len(matches) > 0:
                return [self.childcategories[m] for m in matches]

        raise ValueError("category_id argument must be in childcategories.")

    def browse_path(self, path, parent=None):
        if isinstance(path, str):
            path = path.split("/")
        parent = parent or self.root_category.value
        self.goto(parent)  # Implicitly used by parse_category_id
        path = list(path)  # We need this to be mutable
        if len(path) == 1:  # Base Case
            for node in self.parse_category_id(path[0]):  # Goto each option
                self.goto(node)
                yield self  # and yield
        else:  # Non Base Case
            p = path.pop(0)  # Remove the first element
            for node in self.parse_category_id(p):  # Goto each option
                for i in self.browse_path(
                    path, node
                ):  # Parse remaining path with each option as a parent
                    yield i  # and yield recursively until we get to BaseCase

    def traverse_path(self, path, parent=None):
        if isinstance(path, str):
            path = path.split("/")
        deque(self.browse_path(path, parent))

    @property
    def path(self):
        """Yields self until self gets to defined root category.

        First element is the immediate parent, not current category.
        """
        history = [self.category_id]  # Remember current state
        yield self
        while int(self.category_id) != int(self.root_category.value):
            if len(set(history)) != len(history):
                raise BrowserError(
                    (
                        "Recursive history detected.  You have likely run `goto`"
                        "outside the expected scope of category {}"
                    ).format(self.root_category.value)
                )
            self.goto(self.parent_category_id)
            history.append(self.category_id)
            yield self
        self.goto(history[0])  # return to where we originally were

    @property
    def pathname(self):
        """Get full pathname as a single string."""
        path = map(lambda x: x.name, self.path)
        path = list(reversed(list(path)))
        return path

    def _flag(self, series, meta=dict()):
        """Internal method to add to mark a series for export with some meta.

        Parameters
        ----------
        series : dict,
            an element of a childseries list from Series.query('...')

        Note
        ----
        Does not validate that series ids are valid.
        """
        meta = self._parse_meta(series, meta)
        self.flags[series["series_id"]] = meta

    def flag_re(self, pat, field="name", meta=dict()):
        """
        Parameters
        ----------
        pat : str,
            Regular expression to search for
        field : str, defaults to "name"
            one of "f", "name", "series_id", "units", "updated"
        meta : dict,
            Custom metadata to pass along with data collection.
            Keys must be either str or callable.
            If str, values must be json-serializable.

        Example
        -------
        .. code-block:: python

            def parse_series_name(series_name):
                '''Example callable parsing the series name.

                Returns
                -------
                dict,
                    updates ``meta_dict``
                '''
                return {"meta_1" : series_name.lower(),
                        "meta_2" : series_name.upper()}

            meta = {
                parse_series_name : 'name', # Keys must be a valid fieldname
                "other" : "foo",
                }

            browser_instance.flag_re('capacity', 'name', meta=meta)
        """
        lookup = lambda x: re.search(pat, x[field], flags=re.I)
        results = filter(lookup, self.childseries)
        for r in results:
            self._flag(r, meta)

    def _parse_meta(self, series, meta):
        """Parses a meta-information dictionary with actual values.

        Parameters
        ----------
        meta : dict,
            dict of meta information associated with flagged series_id
        childseries :
            pass
        """
        meta = dict(meta)  # Avoid side effects
        funcs = filter(callable, meta)  # Get any callables
        valid_names = ["f", "name", "series_id", "units", "updated"]
        for f in funcs:
            fieldname = meta.pop(f)
            if fieldname not in valid_names:
                raise ValueError("Meta dict must take one of {}".format(valid_names))
            m = f(series[fieldname])  # This should return a dictionary
            meta.update(m)
        return meta

    def export(self):
        """Export data as a pandas DataFrame."""
        data = api.Series(*filter(None, self.flags.keys()), self.api_key).to_dataframe()
        output = data.merge(
            pd.DataFrame(self.flags).T,
            how="outer",
            left_on="series_id",
            right_index=True,
        )
        self.flush()
        return output

    def flush(self):
        self.flags = dict()


class Electricity(Browser):
    """Electricity Data."""

    root_category = Category.electricity


class SEDS(Browser):
    """State Energy Data System."""

    root_category = Category.seds


class Petroleum(Browser):
    """Petroleum Data."""

    root_category = Category.petroleum


class NaturalGas(Browser):
    """Natural Gas Data."""

    root_category = Category.natural_gas


class TotalEnergy(Browser):
    """Total Energy Data."""

    root_category = Category.total_energy


class Coal(Browser):
    """Coal Data."""

    root_category = Category.coal


class STEO(Browser):
    """Short-Term Energy Outlook."""

    root_category = Category.steo


class AEO(Browser):
    """Annual Energy Outlook."""

    root_category = Category.aeo

    @property
    def scenario(self):
        fullpath = self.pathname
        if len(fullpath) > 2:
            return fullpath[2]
        else:
            return "Not currently within a scenario."

    @property
    def aeoname(self):
        fullpath = self.pathname
        if len(fullpath) > 1:
            return fullpath[1]
        else:
            return "Not currently within a scenario"


class CrudeOil(Browser):
    """Crude Oil Imports."""

    root_category = Category.crude_oil


class InternationalEnergy(Browser):
    """International Energy Data."""

    root_category = Category.international_energy


class USESOD(Browser):
    """U.S. Electric System Operating Data."""

    root_category = Category.usesod


class CO2Emissions(Browser):

    root_category = Category.co2_emissions


class USNuclearOutages(Browser):

    root_category = 2889994



from dynaconf import Dynaconf, constants

config = Dynaconf(
    envvar_prefix="EIA",
    load_dotenv=True,
    warn_dynaconf_global_settings=True,
    environments=True,
    default_env="eia",
    lowercase_read=False,
    default_settings_paths=constants.DEFAULT_SETTINGS_FILES,
)

APIKEY = config.get("APIKEY")



import asyncio
import pandas as pd

from typing import List

from .base import BaseQuery, yield_chunks


class Series(BaseQuery):
    """Retrieve one or more series from the series API.

    Example:

        .. code-block::python

            from eia.api import Series
            
            S = Series(
                "AEO.2015.REF2015.CNSM_DEU_TOTD_NA_DEU_NA_ENC_QBTU.A", 
                "AEO.2015.REF2015.CNSM_ENU_ALLS_NA_DFO_DELV_ENC_QBTU.A",
            )
            S.to_dataframe()
    """

    endpoint = "series"

    def __init__(self, *series_ids: str, apikey: str = None):
        super().__init__(apikey)
        self.series_ids = [
            ";".join(chunk) for chunk in yield_chunks(filter(None, series_ids), 100)
        ]

    async def _get_data(self):
        """Send a request for each batch of series ids and await their results.
        """
        coros = []
        results = []
        for series_ids in self.series_ids:
            response = self._post(data={"series_id": series_ids})
            coros.append(response)
            if len(coros) == 5:  # throttle at 5
                _ = await asyncio.gather(*coros)
                results.extend(_)
                coros = []  # Reset accumulator
        if coros:
            results.extend(await asyncio.gather(*coros))

        return filter(None, results)

    async def parse(self, key) -> List[dict]:
        """
        Collect one dict per series and drop request metadata.
        """
        data = await self._get_data()
        output = []
        for group in data:
            for series in group.get(key, []):
                output.append(series)
        return output

    def to_dict(self) -> List[dict]:
        """Return the input series as a list of dictionaries.
        """
        return asyncio.run(self.parse(key="series"))

    def to_dataframe(self, include_metadata: bool = True) -> pd.DataFrame:
        """Return the input series as a dataframe.
        """
        # Get all our data first with async
        # Note that all our pandas work will tax CPU so we wouldn't expect any
        # performance gains from doing the data parsing as a callback
        records = self.to_dict()
        data = []
        for series in records:
            df = pd.DataFrame(series.pop("data"), columns=["period", "value"])
            if include_metadata:
                df = df.assign(**series)
            data.append(df)
        return pd.concat(data, ignore_index=True)




import asyncio
import pandas as pd

from typing import List

from .base import BaseQuery, yield_chunks
from .series import Series


class SeriesCategory(Series):
    """
    The SeriesCategory endpoint returns all categories that a given series id belongs to.

    Example:

        .. code-block::python

            from eia.api import SeriesCategory

            SeriesCategory(
                "AEO.2015.REF2015.CNSM_DEU_TOTD_NA_DEU_NA_ENC_QBTU.A",
                "AEO.2015.REF2015.CNSM_ENU_ALLS_NA_DFO_DELV_ENC_QBTU.A"
            ).to_dataframe()

    Note:
        Many times, there will be multiple paths to get to a single category.
        A single child series can have multiple nearly identical parents that
        differ on how a user has traversed to that category. This could be useful for
        collecting all relevant "tags" or descriptive data on a series.
    """

    endpoint = "series/categories"

    def to_dict(self) -> List[dict]:
        """Return the input series as a list of dictionaries.
        """
        return asyncio.run(self.parse(key="series_categories"))

    def to_dataframe(self) -> pd.DataFrame:
        """
        Return all category ids and category names associated with each series id as a DataFrame.
        """
        data = self.to_dict()
        return pd.concat(
            [
                pd.DataFrame(series["categories"]).assign(series_id=series["series_id"])
                for series in data
            ],
            ignore_index=True,
        )



import asyncio
import pandas as pd

from .base import BaseQuery


class Relation(BaseQuery):
    """DEPRECATED: This API endpoint does not appear to be accurately documented by EIA.

    It is unclear what this API is supposed to accept or return.

    Example:

        .. code-block::python

            from eia.api import Relation
            Relation("ELEC.GEN.ALL-99.A", "USA-AK",).to_dataframe()
    """

    endpoint = "relation"

    def __init__(self, geoset_id, *regions, api_key: str = None):
        super().__init__(api_key)
        self.geoset_id = geoset_id
        self.regions = list(regions)

    async def _get_data(self):
        data = {"relation_id": self.geoset_id, "region": ",".join(self.regions)}
        return await self._get(data=data)

    def to_dict(self) -> dict:
        """Return the raw geoset response as a python dictionary.
        """
        return asyncio.run(self._get_data())

    def to_dataframe(self, include_metadata: bool = True) -> pd.DataFrame:
        """Return the relation data as a dataframe.
        """
        raise NotImplementedError



"""
Core documentation for the EIA API can be found at: https://www.eia.gov/opendata/commands.php
"""
import abc
import httpx
import itertools

from loguru import logger
from typing import Iterator
from urllib.parse import urljoin

from eia import settings


def yield_chunks(iterator: Iterator, n: int) -> Iterator:
    """Chunk an iterable into iterables of at most size n.
    """
    iterator = iter(iterator)  # Convert whatever we have into an iterator
    for edge in iterator:  # Exit when iterator is exhausted
        boundary = itertools.islice(iterator, 0, n - 1)
        yield list(itertools.chain([edge], boundary))


class BaseQuery(abc.ABC):

    ROOT = "https://api.eia.gov"

    def __init__(self, apikey: str = None):
        self._configure(apikey)

    def _configure(self, apikey: str):
        """Set instance API key, default parameters and base url.
        """
        # Set authentication
        self.apikey = apikey or settings.APIKEY
        assert self.apikey, "Missing required apikey."

        # Set default API parameters
        self._params = {"api_key": self.apikey, "out": "json"}

        # Set the default endpoint
        endpoint = self.endpoint if self.endpoint.endswith("/") else f"{self.endpoint}/"
        self.url = urljoin(self.ROOT, endpoint)

    async def _get(self, data: dict = {}) -> dict:
        """Asynchronously send a GET request and retrieve a python dict.
        """
        params = {**self._params, **data}
        async with httpx.AsyncClient(timeout=10 * 60.0) as client:
            response = await client.get(self.url, params=params)
            logger.debug(
                "Made GET to {url} with parameters\n{params}",
                url=self.url,
                params=params,
            )
        return response.json()

    async def _post(self, data: dict = {}) -> dict:
        """Asynchronously send a POST request and retrieve a python dict.
        """
        async with httpx.AsyncClient(timeout=10 * 60.0) as client:
            response = await client.post(self.url, params=self._params, data=data)
            logger.debug(
                "made POST to {url} with parameters {params} and payload\n{payload}",
                url=self.url,
                params=self._params,
                payload=data,
            )
        return response.json()

    @abc.abstractmethod
    def to_dict(self):
        raise NotImplementedError

    @abc.abstractmethod
    def to_dataframe(self):
        raise NotImplementedError



import re
import httpx
import pandas as pd

from math import ceil
from typing import List

from .base import BaseQuery


def _quotify(value: str):
    """Add literal quotes around a string to format search terms correctly for the search API.
    """
    value = value.strip('"')
    if not re.match(re.escape(f'^"{value}"$'), value):
        value = f'"{value}"'
    return value


def _make_date_range(date_range: List):
    """Create a SOLR friendly date range from pandas-parseable datetimes.
    """
    assert len(date_range) == 2, "Must specify only a begin and end date."
    begin, end = date_range
    eia_date_fmt = "%Y-%m-%dT%H:%M:%SZ"
    daterange = map(
        lambda x: x.strftime(eia_date_fmt), map(pd.to_datetime, (begin, end))
    )
    return f'[{" TO ".join(daterange)}]'


search_term_cleaners = {
    "series_id": _quotify,
    "name": _quotify,
    "last_updated": _make_date_range,
}
"""Registry of cleaning functions to apply to individual search terms.
"""


class Search(BaseQuery):

    endpoint = "search"

    def __init__(self, apikey: str = None):
        super().__init__(apikey)

    def _send_search_query(
        self, search_term, search_value, num_rows: int = 0, chunksize: int = 5000
    ):
        """
        """
        assert (
            search_term in search_term_cleaners
        ), f"Invalid search term {search_term} not in {', '.join(search_term_cleaners.keys())}"
        # Clean search value
        search_value = search_term_cleaners[search_term](search_value)
        if not num_rows:
            # Retrieve actual rows per page
            response = httpx.get(
                self.url,
                params={
                    "rows_per_page": 1,
                    "page_num": 1,
                    "search_value": search_value,
                    "search_term": search_term,
                },
                timeout=10.0,
            )
            response.raise_for_status()
            num_rows = response.json()["response"]["numFound"]
        pages = ceil(num_rows / chunksize)
        search_results = []
        for page in range(pages):
            offset = page * chunksize
            pagesize = chunksize
            if offset + chunksize > num_rows:
                pagesize = num_rows - offset
            params = {
                "page_num": page,
                "rows_per_page": chunksize,
                "search_term": search_term,
                "search_value": search_value,
            }
            results = httpx.get(self.url, params=params, timeout=10.0)
            results.raise_for_status()
            yield results.json()["response"]["docs"]

    def to_dict(self, *args, **kwargs):
        return list(self._send_search_query(*args, **kwargs))

    def to_dataframe(self, *args, **kwargs):
        return pd.concat((chunk for chunk in self._send_search_query(*args, **kwargs)))



from enum import Enum


class Category(Enum):

    root = 371

    electricity = 0

    seds = 40203

    petroleum = 714755

    natural_gas = 714804

    total_energy = 711224

    coal = 717234

    steo = 829714

    aeo = 964164

    crude_oil = 1292190

    international_energy = 2134384

    usesod = 2123635

    co2_emissions = 2251604

    us_nuclear_outages = 2889994



from importlib import metadata
from loguru import logger

__version__ = metadata.version("eia")

# Do not pass logs to application; it is always possible to enable logging
# at the application level, e.g. logger.enable("eia")
logger.disable(__name__)



import asyncio
import pandas as pd

from .base import BaseQuery


class Geoset(BaseQuery):
    """Endpoint to collect a set of series belonging to a geoset in a list of valid regions.

    Example:

        .. code-block:: python

            from eia.api import Geoset
            Geoset("ELEC.GEN.ALL-99.A", "USA-CA", "USA-FL", "USA-MN").to_dataframe()
    """

    endpoint = "geoset"

    def __init__(self, geoset_id: str, *regions: str, api_key: str = None):
        super().__init__(api_key)
        self.geoset_id = geoset_id
        self.regions = list(regions)

    async def _get_data(self):
        """Send a request for each batch of series ids and await their results.
        """
        data = {"geoset_id": self.geoset_id, "regions": ",".join(self.regions)}
        return await self._get(data=data)

    def to_dict(self) -> dict:
        """Return the raw geoset response as a python dictionary.
        """
        return asyncio.run(self._get_data())["geoset"]

    def to_dataframe(self, include_metadata: bool = True) -> pd.DataFrame:
        """Return the geoset data as a dataframe.

        Note:
            
            Only metadata at the series level can be dropped. Geoset-level metadata
            will always be included.
        """
        data = asyncio.run(self._get_data())["geoset"]
        series_data = data.pop("series")
        geoset_meta = dict(data)
        accumulator = []
        for series in series_data.values():
            df = pd.DataFrame(series.pop("data"), columns=["period", "value"])
            if include_metadata:
                df = df.assign(**series)
            accumulator.append(df)
        df = pd.concat(accumulator, ignore_index=True)
        # Reattach the geoset-level data.
        df = df.assign(**geoset_meta)
        return df




from .series import Series
from .geoset import Geoset
from .category import Category
from .series_category import SeriesCategory
from .updates import Updates
from .search import Search

# FIXME:40 The relation API doesn't appear to work at the moment.
# from .relation import Relation

__all__ = [
    "Series",
    "Geoset",
    # "Relation",
    "Category",
    "SeriesCategory",
    "Updates",
    "Search",
]



"""
Note:
    This API might be under-documented because it doesn't seem all that useful in reality.
"""
import asyncio
import pandas as pd

from math import ceil

from .base import BaseQuery


class Updates(BaseQuery):
    """
    Used to poll for the most recent update date of all series belonging to a category.
    Results are by default ordered by the most recent update.

    Example:

        .. code-block:: python

            from eia.api import Updates
            Updates(category_id=711230).to_dict()
    """

    endpoint = "updates"

    def __init__(
        self, category_id: int = None, deep: bool = False, api_key: str = None,
    ):
        super().__init__(api_key)
        self.category_id = category_id
        self.deep = deep

    async def _get_data(self, rows: int = None):
        default_params = {
            "category_id": self.category_id,
            "deep": self.deep,
        }
        max_rows = 10000  # The Updates API will allow up to 10000 rows for each request
        # Figure out how many rows we want to get
        total_rows = (
            await self._get(data={**default_params, "rows": 1, "firstrow": 0})
        )["data"]["rows_available"]
        if rows:
            n_rows = min(rows, total_rows)
        else:
            n_rows = total_rows

        n_pages = int(ceil(n_rows / max_rows))

        coros = []
        for page in range(n_pages):
            page_nrows = min(
                n_rows, max_rows
            )  # Number of rows to collect for this page
            firstrow = page * page_nrows  # 0-indexed means this starts at 0
            page_params = {**default_params, "firstrow": firstrow}
            if firstrow + page_nrows > n_rows:
                page_nrows = n_rows - firstrow
            response = self._get(data={**page_params, "rows": page_nrows})
            coros.append(response)
            coros.append(
                asyncio.sleep(0.25)
            )  # Rate limit ourselves to 4 requests/second
        return await asyncio.gather(*coros)

    async def parse(self, rows: int = None):
        updates = []
        data = await self._get_data(rows)
        for response in filter(None, data):
            for datum in response.get("updates"):
                updates.append(datum)
        return updates

    def to_dict(self, rows: int = None):
        return asyncio.run(self.parse(rows))

    def to_dataframe(self, rows: int = None):
        return pd.DataFrame(self.to_dict(rows))



import httpx
import pandas as pd

from .base import BaseQuery


class Category(BaseQuery):
    """The category endpoint serves to traverse relationships or tags of data from the EIA API.
    """

    endpoint = "category"

    def __init__(self, category_id: int = None, apikey: str = None):
        super().__init__(apikey)
        self.category_id = category_id

    def to_dict(self) -> dict:
        data = {**self._params}
        if self.category_id:
            data["category_id"] = self.category_id
        data = httpx.get(self.url, params=data).json()["category"]
        return data

    @property
    def childcategories(self):
        data = self.to_dict()
        return pd.DataFrame(data["childcategories"]).assign(
            parent_category_id=data["category_id"], parent_category_name=data["name"]
        )

    @property
    def childseries(self):
        data = self.to_dict()
        return pd.DataFrame(data["childseries"]).assign(
            category_id=data["category_id"],
            parent_category_id=data["parent_category_id"],
            category_name=data["name"],
        )

    def to_dataframe(self) -> pd.DataFrame:
        """The category endpoint does not contain any series data.
        """
        raise NotImplementedError




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['eia', 'eia.api']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'dynaconf>=3.0.0,<4.0.0',
 'httpx>=0.13.3,<0.14.0',
 'loguru>=0.5.1,<0.6.0',
 'pandas>=1.1.0,<2.0.0']

setup_kwargs = {
    'name': 'pyeia',
    'version': '0.1.4',
    'description': 'Python client for the Energy Information Administration (EIA) API',
    'long_description': '# Configuration\n\nYou can configure pyeia with your API key either at runtime.\n\n- Declare `EIA_APIKEY="myapikey"` in a `.env` file\n- Set an environment variable explicitly, `export EIA_APIKEY="myapikey"`\n- If you are using dynaconf, you can include an `[eia]` environment in your `settings.toml` file (or any other configured settings files.)\n\n```toml\n[eia]\napikey = "my apikey"\n```\n\n# About\n\nThe U.S. Energy Information Adminsitration provides an API for access to commonly used datasets for policy makers\nand researchers. See the [EIA API documentation](http://www.eia.gov/opendata/commands.cfm) for more information.\n\nWarning : This package is a work in progress!  A substantial update is expected in January 2020, with a published version on PyPi.  The author took a break from this domain area, but is returning!  Hoping to have a similar or identical R interface/API as well, but that may be much farther down the pipeline.\n\n# Basic Usage\n\nSince this package is still under active development, it has not been pushed to PyPi. That said, I believe it is\nstable and reliable enough for immediate use.  You can install this via git+https, i.e. :\n\n```bash\npip install pyeia\npip show pyeia\n```\n\nThere are two main strategies for interacting with this package.\n\n## EIA Browser\n\n[EIA provides a web-based data browser](http://www.eia.gov/opendata/qb.cfm)\nSince most interactions for discovering data via the API will likely occur\nthrough this browser, this motivated a programmatic version.\n\nThe general strategy is to traverse a datapath or multiple datapaths, and\nwhen you arrive to the desired node, you flag one or more dataseries.  \nThere is also the ability to add in meta information as you flag a dataseries.\n\nRunning the `export` method on a Browser object will make a request to the\n`Series` API to collect data you\'ve flagged.\n\nThere\'s currently a separate class for each dataset which is mostly syntactic.\nIn the future, there will likely be methods and visualizations builtin that are\nspecific to the datasets described at the root category level from EIA.\n\n1. [Browser Quickstart to Collect AEO data](examples/aeo_quickstart.py)\n2. [Computing Marginal Values for AEO data](examples/aeo_marginal_values.py)\n\n## Direct API usage\n\nEach endpoint has a corresponding class in `eia.api`.  Every class has a `query` method that makes a call to EIA.\nThe returned result is always the response body.  Metadata about the request is dropped.  The `Series` and `Geoset`\nclasses have a special `query_df` method since their response bodies have a naturally tabular schema.\n\n\n```python\nfrom eia import api\n\nmyapikey = ""  # Register here : www.eia.gov/opendata/register.cfm\n\n# Make a call to the Category endpoint\ncategory = api.Category(myapikey)\ncategory.query()\n\n# Make a call to the Series endpoint\nseries = api.Series(\n    "AEO.2015.REF2015.CNSM_DEU_TOTD_NA_DEU_NA_ENC_QBTU.A",\n    "AEO.2015.REF2015.CNSM_ENU_ALLS_NA_DFO_DELV_ENC_QBTU.A",\n    api_key=myapikey,\n)\nseries.to_dict()  # Export data from its json response\n# Make the same query, but get results as a pandas DataFrame\nseries.to_dataframe()\n\n# Make a call to the Geoset endpoint\ngeoset = api.Geoset("ELEC.GEN.ALL-99.A", "USA-CA", "USA-FL", "USA-MN", api_key=myapikey)\ngeoset.to_dict()\ngeoset.query_df()\n\n# Make a call to the SeriesCategory endpoint\n\nseriescategory = api.SeriesCategory(\n    "AEO.2015.REF2015.CNSM_DEU_TOTD_NA_DEU_NA_ENC_QBTU.A",\n    "AEO.2015.REF2015.CNSM_ENU_ALLS_NA_DFO_DELV_ENC_QBTU.A",\n    api_key=myapikey,\n)\nseriescategory.to_dict()\n\n# Make a call to the Updates endpoint\n\nupdates = api.Updates(\n    category_id=2102358,\n    rows=0,\n    firstrow="currently_not_used",\n    deep=False,\n    api_key=myapikey,\n)\nupdates.to_dict()\n\n# Make a call to the Search endpoint\nsearch = api.Search(api_key=myapikey)\n\n# Make a series_id search\nsearch.to_dict("series_id", "EMI_CO2_COMM_NA_CL_NA_NA_MILLMETNCO2.A", "all")\n\n# Make a name search\nsearch.to_dict("name", "crude oil", 25)\n\n# Make a date-range search\n# Dates can be input as a list/tuple of any valid pd.to_datetime argument\nsearch.to_dict("last_updated", ["Dec. 1st, 2014", "06/14/2015 3:45PM"])\n```\n',
    'author': 'Thomas Tu',
    'author_email': 'thomasthetu@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/thomastu/pyEIA',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
